/*----------------------------------------------------------------------*/
/* send messages with ISDN using MSN & CLI                              */
/*                                                                      */
/* Author:      Giuseppe Zanetti <beppe@profuso.com>                    */
/*----------------------------------------------------------------------*/

#include <stdio.h>
#include <errno.h>
#include <getopt.h>
#include <signal.h>
#include <string.h>
#include <fcntl.h>
#include <termios.h>
#include <unistd.h>
#include <sys/file.h>
#include <sys/ioctl.h>
#include <asm/ioctls.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <sys/types.h>

/*----------------------------------------------------------------------*/
/* Exit with an error                                                   */
/*----------------------------------------------------------------------*/

void fatal(char *error)
{
	fprintf(stderr,"ERROR %s\n", error);
	exit(error);
}

/*----------------------------------------------------------------------*/
/* Pause for "sec" seconds.                                                 */
/*----------------------------------------------------------------------*/

int pausa(int sec)
{
	struct timeval tv;

	/* Pause for n seconds. */
	tv.tv_sec = sec;
	tv.tv_usec = 0;
	if (select(0, NULL, NULL, NULL, &tv) == -1) fatal("ERR_SELECT");
}

/*----------------------------------------------------------------------*/
/* linres: Resets the (modem) serial line to sane, raw value            */
/* Also drops DTR line for 3 seconds.                                   */
/*                                                                      */
/* Author:	Niccolo Rigacci <fd131@cleveland.freenet.edu>               */
/*                                                                      */
/* Copyright (C) 1996 Niccolo Rigacci                                   */
/*----------------------------------------------------------------------*/

void linres(char *tty)
{
	int fd, line;
	struct timeval tv;
	struct termios ts;

	/* Open the modem device and lock it. */
	if ((fd = open(tty, O_RDWR | O_NONBLOCK)) == -1) fatal("DEVICE");
	if (flock(fd, LOCK_EX | LOCK_NB) == -1) fatal("ERR_LOCK");

	/* Flush input and output queues. */
	if (ioctl(fd, TCFLSH, 2) != 0) fatal("ERR_IOCTL");

	/* Turn off DTR control line. */
	line = TIOCM_DTR;
	if (ioctl(fd, TIOCMBIC, &line) != 0) fatal("ERR_IOCTL");

	/* Pause for 3 seconds. */
	pausa(3);

	/* Turn on DTR control line. */
	line = TIOCM_DTR;
	if (ioctl(fd, TIOCMBIS, &line) != 0) fatal("ERR_IOCTL");

	/* Fetch the current terminal parameters. */
	if (ioctl(fd, TCGETS, &ts) != 0) fatal("ERR_IOCTL");

	/* Sets hardware control flags:				*/
	/* 38400 baud						*/
	/* 8 data bits						*/
	/* Enable receiver						*/
	/* Ignore CD (local connection)				*/
	/* Use RTS/CTS flow control					*/

	ts.c_cflag = B38400 | CS8 | CREAD | CLOCAL | CRTSCTS;
	ts.c_iflag = 0;
	ts.c_oflag = NL0 | CR0 | TAB0 | BS0 | VT0 | FF0;
	ts.c_lflag = 0;

	/*--------------------------------------------------------------*/
	/* All these capabilities are turned off; see termios(2)	*/
	/* and stty(1L) man pages:					*/
	/*								*/
	/* c_cflag	!CSTOPB, !PARENB, !HUPCL			*/
	/*								*/
	/* c_iflag	!IGNBRK, !BRKINT, !IGNPAR, !PARMRK, !INPCK,	*/
	/*		!ISTRIP, !INLCR,  !IGNCR,  !ICRNL,  !IXON,	*/
	/*		!IXOFF,  !IUCLC,  !IXANY,  !IMAXBEL		*/
	/*								*/
	/*								*/
	/* ts.c_oflag	!OPOST,   !OLCUC, !OCRNL,  !ONLCR,  !ONOCR	*/
	/*		!ONLRET,  !OFILL, !OFDEL			*/
	/*								*/
	/* ts.c_lflag	!ISIG    !ICANON  !IEXTEN !ECHO  !ECHOE		*/
	/*		!ECHOK   !ECHONL  !NOFLSH !XCASE !TOSTOP	*/
	/*		!ECHOPRT !ECHOCTL !ECHOKE			*/
	/*--------------------------------------------------------------*/

	ts.c_cc[VINTR]    = '\0';
	ts.c_cc[VQUIT]    = '\0';
	ts.c_cc[VERASE]   = '\0';
	ts.c_cc[VKILL]    = '\0';
	ts.c_cc[VEOF]     = '\0';
	ts.c_cc[VTIME]    = '\0';
	ts.c_cc[VMIN]     = 1;
	ts.c_cc[VSWTC]    = '\0';
	ts.c_cc[VSTART]   = '\0';
	ts.c_cc[VSTOP]    = '\0';
	ts.c_cc[VSUSP]    = '\0';
	ts.c_cc[VEOL]     = '\0';
	ts.c_cc[VREPRINT] = '\0';
	ts.c_cc[VDISCARD] = '\0';
	ts.c_cc[VWERASE]  = '\0';
	ts.c_cc[VLNEXT]   = '\0';
	ts.c_cc[VEOL2]    = '\0';

	/* Sets the new terminal parameters. */
	if (ioctl(fd, TCSETS, &ts) != 0) fatal("ERR_IOCTL");

	/* Release the modem device. */
	if (flock(fd, LOCK_UN) == -1) fatal("ERR_UNLOCK");
	if (close(fd) != 0) fatal("ERR_CLOSE");
}

/*----------------------------------------------------------------------*/
/* Send a string to modem                                               */
/*----------------------------------------------------------------------*/

msend(int fd, char *cmd)
{
	if (write(fd, cmd, strlen(cmd)) != strlen(cmd)) fatal("ERR_WRITE");
	printf("WRITE: %s\n",cmd);
}

/*----------------------------------------------------------------------*/
/* Receive a string from modem                                          */
/*----------------------------------------------------------------------*/

mgets(int fd, char *buf)
{
	int r;
	char s[2];

	strcpy(buf,"");

	while(1)
	{
		r=read(fd, s, 1);
		if(r<0) break;
		s[1]='\0';
		if(s[0]==10) break;
		if(s[0]!=13) strcat(buf,s);
	}

	/* printf("READ: %d: %s\n",r,buf); */

	return(r);
}

mflush(int fd)
{
	int r;
	char s[2];

	while(1)
	{
		r=read(fd, s, 1);
		if(r<0) break;
		s[1]='\0';
		printf("%c",s[0]);
	}
}

/*----------------------------------------------------------------------*/
/* Decimal to binary conversione (very bad code, but was fast to write) */
/*----------------------------------------------------------------------*/

dec2bin(char*buffer, char c)
{
	int n;
	int r;
	char tmp[1024];
	char reverse[1024];
	int i;

	strcpy(buffer,"");
	strcpy(reverse,"");
	n=c;

	while(1)
	{
		r=n % 2;
		n=n / 2;
		sprintf(tmp,"%d",r);
		strcat(reverse,tmp);

		if(n<=0) break;
	}
	strcat(reverse,"0000000000000000000000");

	for(i=0;i<8;i++) buffer[i]=reverse[7-i];
	buffer[8]='\0';
}

/*----------------------------------------------------------------------*/
/* Send messages using the ISDN modem emulator                          */
/*----------------------------------------------------------------------*/

main(int argc, char *argv[])
{
	char msn[255][100];
	char tty[1024];
	char buf[1024];
	char cmd[1024];
	char tel[1024];
	char tmp[1024];
	char sms[1024];
	int fd;
	int i,j;
	int n;
	int e;
	int carattere;
	FILE *f;

	f=fopen("msn.conf","r");
	if(f==NULL)
	{
		fprintf(stderr,"Non posso aprire msn.conf\n");
		exit(1);
	}
	else
	{
		fscanf(f,"%s",msn[0]);
		fscanf(f,"%s",msn[1]);
		fclose(f);
	}

	if(argc<3)
	{
		fprintf(stderr,"Usage: %s tty phone\n",argv[0]);
		exit(0);
	}

	strcpy(tty,argv[1]);
	strcpy(tel,argv[2]);

	/* reset the modem device */
	linres(tty);

	/* open modem device */
	if ((fd = open(tty, O_RDWR | O_NONBLOCK)) == -1) fatal("ERR_DEVICE");
	if (flock(fd, LOCK_EX | LOCK_NB) != 0) fatal("ERR_LOCK");

	/* reset the modem */
	msend(fd,"ATZ\r"); mflush(fd);

	/* set my MSN */
	sprintf(cmd,"AT&E%s",tel);
	msend(fd,cmd); mflush(fd);

	/* put in data mode */
	msend(fd,"ATS18=4\r"); mflush(fd);

	/* present the CLI only one time */
	msend(fd,"ATS13.4=1\r"); mflush(fd);

	while(1)
	{
		carattere=0;
		for(e=7;e>=0;e--)
		{
			while(1)
			{
				mgets(fd, buf);
				if(strstr(buf,"CALLER")!=NULL)
				{
					strcpy(tmp,strchr(buf,':')+2);
					for(i=0;i<=1;i++) if(strcmp(msn[i],tmp)==0) break;
					printf("%d",i); fflush(stdout);
					carattere += i*1<<e;
					break;
				}
			}
		}
		printf(" => %c\n",carattere);
	}


	/* Close modem device. */
	if (flock(fd, LOCK_UN) == -1) fatal("ERR_UNLOCK");
	close(fd);
}

